import requests, zipfile, platform , rich
from pathlib import Path
from tqdm import tqdm
from rich.console import Console
from time import sleep
import shutil
def ffmpeg():
    
    console=Console()

    if platform.system()!= "Windows" and shutil.which("ffmpeg") is None: 
       print("Windows only is supported for ffmpeg installation")
       print(" plz download ffmpeg from https://www.gyan.dev/ffmpeg/builds/ffmpeg-release-essentials.zip")
       exit()

    FFmpeg_installation= Path.home()/".fid-ffmpeg"
    FFmpeg_installation.mkdir(exist_ok=True)
    zip=FFmpeg_installation/"ffmpeg.zip"
    exe=FFmpeg_installation/"ffmpeg.exe"
    
    with console.status("checking for ffmpeg....") as status:
        sleep(2)
        if exe.exists():
            print("ffmpeg already exists")
            return str(exe)
        elif shutil.which("ffmpeg") is not None: 
            print("ffmpeg already exists")
            return shutil.which("ffmpeg")
        else:
            print("\nffmpeg not found, downloading it ....\n")

    r =requests.get("https://www.gyan.dev/ffmpeg/builds/ffmpeg-release-essentials.zip",stream=True)
    total = int(r.headers.get("content-length"))
    with open(zip,"wb")as f:
         with tqdm(total=total,unit="B",unit_scale=True,colour="green", bar_format="{l_bar}{bar} | {n_fmt}/{total_fmt}" ) as bar:
            for chunk in r.iter_content(1024*1024):
                f.write(chunk)
                bar.update(len(chunk))
    with console.status("extracting....") as status:
        sleep(1.5)
        with zipfile.ZipFile(zip)as z:
            for name in z.namelist():
                if name.endswith("ffmpeg.exe"):
                   extract= z.extract(name , FFmpeg_installation)
                   Path(extract).rename(exe)
                   print("ffmpeg installed .")
                   break
    return str(exe)

def ckvideo(cPath):
    if not cPath.exists() or not cPath.is_file() or cPath.suffix.lower() not in [".mp4",".avi",".mkv",".mov",".flv",".wmv",".webm"]:  
        print("incorrect video path or unsupported video fromat")
        exit()



import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo


def compress(cPath: Path, crf: int,preset: str, audio_bitrate: str):
    ffmpeg()
    ckvideo(cPath)
    compress_out= cPath.with_stem(f"{cPath.stem}_compressed").with_suffix(".mp4")
    subprocess.run(
            [ffmpeg(), "-i", str(cPath),"-c:v", "libx265", "-crf", str(crf), "-preset", preset,"-vf", "hqdn3d=1.5:1.5:6:6,eq=contrast=1.05:brightness=0.01:saturation=1.05","-pix_fmt","yuv420p","-tag:v", "hvc1","-c:a","aac","-b:a",audio_bitrate,"-movflags", "+faststart","-y",str(compress_out),]
            , check=True,stdout=subprocess.DEVNULL)
def compress_main(app: typer.Typer):
    app.command()(compress)


import questionary
import typer
from .denoise import denoise
from .equalizer import equalizer
#from .compressor import compress
#from .codec import codec
#from .bitrate import bitrate
#from .channels import channels
#from .delay import delay
#from .fade import fade
#from .trim import trim
#from .replace import replace
from .mute import mute
#from .mix import mix
#from .speed import speed
#from .normalize import normalize
#from .volume import volume
from .enhance_audio import enhance_voice

def audio_main(cPath):

    while True:
        print(f"""
╔══════════════════════════════════════╗
║          AUDIO EDITING MENU          ║
╚══════════════════════════════════════╝
""")

        choice= questionary.select(
           "select the editing option you want:",
          choices=[
                "enhance voice",
                "volume up/down",
                "mute audio",
                "normalize audio",
                "speed up/down",
                "decrease noise",
                "mix two audio files",
                "compress audio file",
                "equalize audio",
                "change codec",
                "change bitrate",
                "change channels",
                "add fade in/out",
                "trim audio",
                "replace audio",
                "delay audio",
                "Back to main menu",
                "exit"
            ]).ask()

        if choice is None:
            raise typer.Exit()
  
        if choice=="decrease noise":
            denoise(cPath)
        elif choice=="enhance voice":
            enhance_voice(cPath)
        elif choice=="mute audio":
            mute(cPath)

        elif choice=="normalize audio":
            normalize(cPath)

        elif choice=="mix two audio files":
            mix(cPath)

        elif choice=="compress audio file":
            compress(cPath)

        elif choice=="equalize audio":
            equalizer(cPath)

        elif choice=="change codec":
            codec(cPath)

        elif choice=="change bitrate":
            bitrate(cPath)

        elif choice=="change channels":
            channels(cPath)

        elif choice=="add fade in/out":
            fade(cPath)

        elif choice=="trim audio":
            trim(cPath)

        elif choice=="replace audio":
            replace(cPath)

        elif choice=="delay audio":
            delay(cPath)

        elif choice=="Back to main menu":
            return

        elif choice=="exit":
           raise typer.Exit()


import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo

def enhance_voice(cPath: Path):
    ffmpeg()
    ckvideo(cPath)
    enhance_out=cPath.with_stem(f"{cPath.stem}_enhanced").with_suffix(cPath.suffix)
    subprocess.run([ffmpeg(),"-i",str(cPath),"-af","afftdn,equalizer=f=3000:t=q:w=1:g=5","-y",str(enhance_out)],check=True)
def enhance_main(app:typer.Typer):
    app.command()(enhance_voice)


import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo

def equalizer(cPath: Path , F=None,W=None,G=None,band=None):
    ffmpeg()
    ckvideo(cPath)
    equalizer_out= cPath.with_stem(f"cPath.stem)_equaized")
    subprocess.run([
        ffmpeg(),"-i",str(cPath),"-af",filter_str,str(equalizer_out)], check=True)
def equalizer_main(app: typer.Typer):
    app.command()(equalizer)


import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo


def audio(cPath: Path):
    ffmpeg()
    ckvideo(cPath)
    audio_out=cPath.with_suffix(".mp3")
    subprocess.run([ffmpeg(), "-i", str(cPath), "-vn", "-acodec", "libmp3lame", "-y", str(audio_out)], check=True)
def audio_main(app: typer.Typer):
    app.command()(audio)



from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from rich.table import Table
from rich.align import Align
from rich import box
import pyfiglet

console = Console()

def welcome():
    ascii = pyfiglet.figlet_format("fid-ffmpeg", font="slant")
    console.print(Text(ascii, style="green bold"))
    video_panel = Panel(
        "- Compress video\n"
        "- Create GIF\n",title="Video Options",border_style="green",box=box.ROUNDED,)
    audio_panel = Panel(
        "- Extract audio\n"
        "- Normalize audio\n"
        "- Noise reduction",
        title="Audio Options",
        border_style="green",
        box=box.ROUNDED,)
    stream_panel = Panel(
        "Coming Soon",
        title="Stream Options",
        border_style="green",
        box=box.ROUNDED,)
    extra_panel = Panel(
        "- Extract frames\n"
        "- Extract audio",
        title="Extra Options",
        border_style="green",
        box=box.ROUNDED,)
    encode_panel = Panel(
        "Coming Soon",
        title="Encode Options",
        border_style="green",
        box=box.ROUNDED,)
    capcut_panel = Panel(
        "[bold]Enhance Audio[/bold]\n"
        " Noise reduction\n",title="CapCut Pro",border_style="white",box=box.ROUNDED,)

    left_grid = Table.grid(padding=(1, 2))
    left_grid.add_row(video_panel, audio_panel)
    left_grid.add_row(extra_panel, stream_panel)
    left_grid.add_row(encode_panel, "")
    interactive = Panel(left_grid,title="Interactive Options",border_style="dim",box=box.DOUBLE,padding=(1, 2),)
    main_layout = Table.grid(padding=(2, 4))
    main_layout.add_row(interactive, Align.center(capcut_panel))
    console.print(Align.left(main_layout))
    console.print(Align.left(Panel("Usage: fid <input_file>",border_style="dim",)))


import questionary
import typer
from .tasks.audio.audio_interactive import audio_main
from .tasks.encode.encode_interactive import encode_main
from .tasks.extract.extract_interactive import extract_main
from .tasks.stream.stream_interactive import stream_main
from .tasks.video.video_interactive import video_main

def fid_main(cPath):

    while True:
        choice= questionary.select(
           "select the editing option you want:",
          choices=[
                "video editing",
                "audio editing",               
                "extract from the video",
                "streaming options",
                "encoding options",
                "exit"
            ]).ask()
            
        if choice is None:
            raise typer.Exit()

        if choice=="video editing":
           video_main(cPath)

        elif choice=="audio editing":
            audio_main(cPath)
        
        elif choice=="extract from the video":
             extract_main(cPath)

        elif choice=="streaming options":
            stream_main(cPath)
        
        elif choice=="encoding options":
            encode_main(cPath)

        elif choice=="exit":
            raise typer.Exit()



import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo


def resize(cPath: Path, width: int):
    ffmpeg()
    ckvideo(cPath)
    resize_out= cPath.with_stem(f"{cPath.stem}_{width}w").with_suffix(".mp4")
    subprocess.run(
            [ffmpeg(),
                "-i", str(cPath),
                "-vf", f"scale={width}:-1",
                "-c:v", "libx264",
                "-preset", "medium",
                "-c:a", "copy",  
                "-y",
                str(resize_out)], check=True)

def resize_main(app: typer.Typer):
    app.command()(resize)



import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo


def gif(cPath: Path):
    ffmpeg()
    ckvideo(cPath)
    gif_out=cPath.with_suffix(".gif")
    subprocess.run([ffmpeg(), "-i", str(cPath), "-t", "3", "-vf", "scale=320:-1", "-y", str(gif_out)], check=True)

def gif_main(app: typer.Typer):
    app.command()(gif)



import questionary
import typer
from .audio import audio
from .frames import frames
#from .thumbnails import thumbnails
#from .keyframes import keyframes
#from .audio_track import audio_track
#from .audio_channels import audio_channels
#from .subtitles import subtitles
#from .subtitles_track import subtitles_track
#from .subtitles_convert import subtitles_convert
#from .chapters import chapters
#from .attachments import attachments

def extract_main(cPath):

    while True:
        print(f"""
╔══════════════════════════════════════╗
║          EXTRACTING MENU          ║
╚══════════════════════════════════════╝
""")

        choice= questionary.select(
           "select the editing option you want:",
          choices=[
                "extract frames",
                "extract audio",
                "extract subtitles",
                "extract chapters",
                "extract thumbnails",
                "extract keyframes",
                "extract audio_track",
                "extract audio_channels",
                "extract subtitles_track",
                "subtitles convert",
                "extract attachments",
                "Back to main menu",
                "exit"
            ]).ask()
        if choice is None:
            raise typer.Exit()

        if choice=="extract audio":
           audio(cPath)

        elif choice=="extract frames":
            frames(cPath)

        elif choice=="Back to main menu":
            return

        elif choice=="exit":
           raise typer.Exit()


import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo


def mute(cPath: Path):
    ffmpeg()
    ckvideo(cPath)
    mute_out=cPath.with_stem(f"{cPath.stem}_muted").with_suffix(cPath.suffix)
    subprocess.run([ffmpeg(), "-i", str(cPath), "-c", "copy", "-an", "-y", str(mute_out)], check=True)

def mute_main(app: typer.Typer):
    app.command()(mute)



import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo

def denoise(cPath: Path):
    ffmpeg()
    ckvideo(cPath)
    denoise_out= cPath.with_stem(f"{cPath.stem}_denoise")
    subprocess.run([ffmpeg(),"-i",str(cPath),"-af","afftdn",str(denoise_out)],check=True)

def denoise_main(app: typer.Typer):
    app.command()(denoise)


import typer
import subprocess
from pathlib import Path
from ..initial_files.error_handling import ffmpeg , ckvideo


def info(cPath: Path):
    ffmpeg()
    ckvideo(cPath)
    ffprobe=Path(ffmpeg()).parent / "ffprobe.exe" if platform.system() == "windows" else "ffprobe"
    subprocess.run([str(ffprobe), "-v", "error", "-show_format", "-show_streams", str(cPath)], check=True)
def info_main(app: typer.Typer):
    app.command()(info)



import typer
import questionary
from pathlib import Path
from .welcome import welcome
from .fid_interactive import fid_main
from .tasks.extract.audio import audio_main
from .tasks.extract.frames import frames_main
from .tasks.audio.mute import mute_main
from .tasks.video.compressor import compress_main
from .tasks.video.gif import gif_main
from .tasks.info import info_main
from .initial_files.error_handling import ffmpeg

app= typer.Typer()

@app.callback(invoke_without_command=True)
def start(ctx : typer.Context):
    if ctx.invoked_subcommand is not None:
        return
       
    ffmpeg()    
    welcome() 

    while True:
        video_path= questionary.path("enter the path to your video:").ask()
        if video_path is None:
            raise typer.Exit()

        cPath= Path(video_path.strip('"').strip("'").replace("\\","/"))
        if cPath.is_file():
            fid_main(cPath)
            break
        else:
            print(f" '{video_path}' is not a valid file path. Please try again") 

audio_main(app)
compress_main(app)
frames_main(app)
gif_main(app)
info_main(app)
mute_main(app)

if __name__=="__main__":
        app()



import questionary
import typer
#from .h264 import h264
#from .h265 import h265
#from .av1 import av1

def encode_main(cPath):

    while True:
        print(f"""
╔══════════════════════════════════════╗
║          ENCODING MENU          ║
╚══════════════════════════════════════╝
""")

        choice= questionary.select(
           "select the encoding option you want:",
          choices=[
                "encode to h264",
                "encode to h265",
                "encode to av1",
                "Back to main menu",
                "exit"
            ]).ask()
        if choice is None:
            raise typer.Exit()
            
       
        elif choice=="Back to main menu":
            return

        elif choice=="exit":
           raise typer.Exit()


import questionary
import typer
from .compressor import compress
#from .concat import concat
#from .crop import crop
#from .fps import fps
from .gif import gif
from .resize import resize
#from .rotate import rotate
#from .speed import speed
#from .trim import trim


def video_main(cPath):

    while True:
        print(f"""
╔══════════════════════════════════════╗
║          VIDEO EDITING MENU          ║
╚══════════════════════════════════════╝
""")
        choice= questionary.select(
           "select the editing option you want:",
          choices=[
                "compress the video",
                "make gif",
                "speed up/down",
                "change fps",
                "concat videos",
                "crop video",
                "resize video",
                "rotate video",
                "trim video",
                "Back to main menu",
                "exit"
            ]).ask()

        if choice is None:
            raise typer.Exit()
            
        if choice=="compress the video":
            compress_choice = questionary.select(
                "Choose compression option:",
                choices=[
                    "smallest size",
                    "medium size (recommended)",
                    "high quality",
                    "Back to main menu",
                    "exit"
                ]
                ).ask()

            if compress_choice is None:
                raise typer.Exit()
            if compress_choice == "Back to main menu":
                continue
            if compress_choice == "exit":
                raise typer.Exit()

            if compress_choice == "smallest size": 
               compress(cPath, crf=33, preset="slower", audio_bitrate="64k")
            elif compress_choice == "medium size (recommended)":
                compress(cPath, crf=27, preset="medium", audio_bitrate="96k")
            elif compress_choice == "high quality":
                compress(cPath, crf=21, preset="medium", audio_bitrate="128k")
     
        elif choice=="make gif":
            gif(cPath)
        
        elif choice=="Back to main menu":
            return

        elif choice=="exit":
           raise typer.Exit()


import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo


def frames(cPath: Path):
    ffmpeg()
    ckvideo(cPath)
    Fdir= cPath.parent
    frames_out= Fdir / "Frames" / cPath.stem
    frames_out.mkdir(parents=True,exist_ok=True)
    subprocess.run([ffmpeg(), "-i", str(cPath),str(frames_out/ "frame_%02d.png")],check=True )
def frames_main(app: typer.Typer):
    app.command()(frames)



import questionary
import typer
#from .hls import hls
#from .rtmp import rtmp
#from .dash import dash
#from .srt import srt
#from .udp import udp
#from .rtsp import rtsp
#from .http import http

def stream_main(cPath):

    while True:
        print(f"""
╔══════════════════════════════════════╗
║          STREAMING MENU          ║
╚══════════════════════════════════════╝
""")

        choice= questionary.select(
           "select the streaming option you want:",
          choices=[
                "stream with hls",
                "stream with rtmp",
                "stream with dash",
                "stream with srt",
                "stream with udp",
                "stream with rtsp",
                "stream with http",
                "Back to main menu",
                "exit"
            ]).ask()
            
        if choice is None:
            raise typer.Exit()
        
        elif choice=="Back to main menu":
            return

        elif choice=="exit":
           raise typer.Exit()


import typer
import subprocess
from pathlib import Path
from ...initial_files.error_handling import ffmpeg , ckvideo

def normalize(cPath: Path):
    ffmpeg()
    ckvideo(cPath)
    normalize_out=cPath.with_stem(f"{cPath.stem}_normalized").with_suffix(cPath.suffix)
    subprocess.run([ffmpeg(),"-i",str(cPath),"-af","loudnorm","-y",str(normalize_out)],check=True)

def normalized_main(app:typer.Typer):
    app.command()(normalize)